@@ -1,9 +1,8 @@
 from __future__ import unicode_literals
 import pytest
 
-import redis3
-from redis3._compat import unichr, unicode
-from .conftest import wait_for_command
+import redis
+from redis._compat import unichr, unicode
 
 
 class TestPipeline(object):
@@ -30,16 +29,6 @@ class TestPipeline(object):
                     [(b'z1', 2.0), (b'z2', 4)],
                 ]
 
-    def test_pipeline_memoryview(self, r):
-        with r.pipeline() as pipe:
-            (pipe.set('a', memoryview(b'a1'))
-                 .get('a'))
-            assert pipe.execute() == \
-                [
-                    True,
-                    b'a1',
-                ]
-
     def test_pipeline_length(self, r):
         with r.pipeline() as pipe:
             # Initially empty.
@@ -84,7 +73,7 @@ class TestPipeline(object):
             pipe.multi()
             pipe.set('a', int(a) + 1)
 
-            with pytest.raises(redis3.WatchError):
+            with pytest.raises(redis.WatchError):
                 pipe.execute()
 
             assert r['a'] == b'bad'
@@ -106,7 +95,7 @@ class TestPipeline(object):
 
             # we can't lpush to a key that's a string value, so this should
             # be a ResponseError exception
-            assert isinstance(result[2], redis3.ResponseError)
+            assert isinstance(result[2], redis.ResponseError)
             assert r['c'] == b'a'
 
             # since this isn't a transaction, the other commands after the
@@ -122,7 +111,7 @@ class TestPipeline(object):
         r['c'] = 'a'
         with r.pipeline() as pipe:
             pipe.set('a', 1).set('b', 2).lpush('c', 3).set('d', 4)
-            with pytest.raises(redis3.ResponseError) as ex:
+            with pytest.raises(redis.ResponseError) as ex:
                 pipe.execute()
             assert unicode(ex.value).startswith('Command # 3 (LPUSH c 3) of '
                                                 'pipeline caused error: ')
@@ -163,22 +152,7 @@ class TestPipeline(object):
         with r.pipeline() as pipe:
             # the zrem is invalid because we don't pass any keys to it
             pipe.set('a', 1).zrem('b').set('b', 2)
-            with pytest.raises(redis3.ResponseError) as ex:
-                pipe.execute()
-
-            assert unicode(ex.value).startswith('Command # 2 (ZREM b) of '
-                                                'pipeline caused error: ')
-
-            # make sure the pipe was restored to a working state
-            assert pipe.set('z', 'zzz').execute() == [True]
-            assert r['z'] == b'zzz'
-
-    def test_parse_error_raised_transaction(self, r):
-        with r.pipeline() as pipe:
-            pipe.multi()
-            # the zrem is invalid because we don't pass any keys to it
-            pipe.set('a', 1).zrem('b').set('b', 2)
-            with pytest.raises(redis3.ResponseError) as ex:
+            with pytest.raises(redis.ResponseError) as ex:
                 pipe.execute()
 
             assert unicode(ex.value).startswith('Command # 2 (ZREM b) of '
@@ -214,7 +188,7 @@ class TestPipeline(object):
             r['b'] = 3
             pipe.multi()
             pipe.get('a')
-            with pytest.raises(redis3.WatchError):
+            with pytest.raises(redis.WatchError):
                 pipe.execute()
 
             assert not pipe.watching
@@ -227,7 +201,7 @@ class TestPipeline(object):
             pipe.watch('a', 'b')
             r['b'] = 3
             pipe.multi()
-            with pytest.raises(redis3.WatchError):
+            with pytest.raises(redis.WatchError):
                 pipe.execute()
 
             assert not pipe.watching
@@ -244,40 +218,6 @@ class TestPipeline(object):
             pipe.get('a')
             assert pipe.execute() == [b'1']
 
-    def test_watch_exec_no_unwatch(self, r):
-        r['a'] = 1
-        r['b'] = 2
-
-        with r.monitor() as m:
-            with r.pipeline() as pipe:
-                pipe.watch('a', 'b')
-                assert pipe.watching
-                a_value = pipe.get('a')
-                b_value = pipe.get('b')
-                assert a_value == b'1'
-                assert b_value == b'2'
-                pipe.multi()
-                pipe.set('c', 3)
-                assert pipe.execute() == [True]
-                assert not pipe.watching
-
-            unwatch_command = wait_for_command(r, m, 'UNWATCH')
-            assert unwatch_command is None, "should not send UNWATCH"
-
-    def test_watch_reset_unwatch(self, r):
-        r['a'] = 1
-
-        with r.monitor() as m:
-            with r.pipeline() as pipe:
-                pipe.watch('a')
-                assert pipe.watching
-                pipe.reset()
-                assert not pipe.watching
-
-            unwatch_command = wait_for_command(r, m, 'UNWATCH')
-            assert unwatch_command is not None
-            assert unwatch_command['command'] == 'UNWATCH'
-
     def test_transaction_callable(self, r):
         r['a'] = 1
         r['b'] = 2
@@ -316,7 +256,7 @@ class TestPipeline(object):
             pipe.llen('a')
             pipe.expire('a', 100)
 
-            with pytest.raises(redis3.ResponseError) as ex:
+            with pytest.raises(redis.ResponseError) as ex:
                 pipe.execute()
 
             assert unicode(ex.value).startswith('Command # 1 (LLEN a) of '
@@ -331,7 +271,7 @@ class TestPipeline(object):
             pipe.llen(key)
             pipe.expire(key, 100)
 
-            with pytest.raises(redis3.ResponseError) as ex:
+            with pytest.raises(redis.ResponseError) as ex:
                 pipe.execute()
 
             expected = unicode('Command # 1 (LLEN %s) of pipeline caused '
